@@ -52,6 +52,9 @@
 #include "tcp.h"
 #include "virt.h"
 #include "libcman.h"
+#include "debug.h"
+
+#define LIBVIRT_XEN_URI "xen:///"
 
 static int running = 1;
 
@@ -188,7 +191,7 @@ wait_domain(fence_req_t *req, virConnectPtr vp, int timeout)
 		sleep(1);
 		vdp = get_domain(req, vp);
 		if (!vdp) {
-			dprintf(2, "Domain no longer exists\n");
+			dbg_printf(2, "Domain no longer exists\n");
 			response = 0;
 			break;
 		}
@@ -198,12 +201,12 @@ wait_domain(fence_req_t *req, virConnectPtr vp, int timeout)
 		virDomainFree(vdp);
 
 		if (di.state == VIR_DOMAIN_SHUTOFF) {
-			dprintf(2, "Domain has been shut off\n");
+			dbg_printf(2, "Domain has been shut off\n");
 			response = 0;
 			break;
 		}
 		
-		dprintf(4, "Domain still exists (state %d) after %d seconds\n",
+		dbg_printf(4, "Domain still exists (state %d) after %d seconds\n",
 			di.state, tries);
 
 		if (++tries >= timeout)
@@ -220,20 +223,20 @@ int
 do_fence_request_tcp(fence_req_t *req, fence_auth_type_t auth,
 		     void *key, size_t key_len, virConnectPtr vp)
 {
-	int fd, ret = -1;
+	int fd = -1, ret = -1;
 	virDomainPtr vdp;
 	char response = 1;
 	char *domain_desc, *domain_desc_sanitized;
 	size_t sz;
 
 	if (!(vdp = get_domain(req, vp))) {
-		dprintf(2, "Could not find domain: %s\n", req->domain);
+		dbg_printf(2, "Could not find domain: %s\n", req->domain);
 		goto out;
 	}
 
 	fd = connect_tcp(req, auth, key, key_len);
 	if (fd < 0) {
-		dprintf(2, "Could call back for fence request: %s\n", 
+		dbg_printf(2, "Could call back for fence request: %s\n", 
 			strerror(errno));
 		goto out;
 	}
@@ -246,7 +249,7 @@ do_fence_request_tcp(fence_req_t *req, fence_auth_type_t auth,
 	case FENCE_OFF:
 		printf("Destroying domain %s...\n", (char *)req->domain);
 
-		dprintf(2, "[OFF] Calling virDomainDestroy\n");
+		dbg_printf(2, "[OFF] Calling virDomainDestroy\n");
 		ret = virDomainDestroy(vdp);
 		if (ret < 0) {
 			printf("virDomainDestroy() failed: %d\n", ret);
@@ -265,8 +268,8 @@ do_fence_request_tcp(fence_req_t *req, fence_auth_type_t auth,
 		domain_desc = virDomainGetXMLDesc(vdp, 0);
 
 		if (domain_desc) {
-			dprintf(3, "[[ XML Domain Info ]]\n");
-			dprintf(3, "%s\n[[ XML END ]]\n", domain_desc);
+			dbg_printf(3, "[[ XML Domain Info ]]\n");
+			dbg_printf(3, "%s\n[[ XML END ]]\n", domain_desc);
 
 			sz = 0;
 			if (cleanup_xml(domain_desc,
@@ -275,14 +278,14 @@ do_fence_request_tcp(fence_req_t *req, fence_auth_type_t auth,
 				domain_desc = domain_desc_sanitized;
 			}
 
-			dprintf(3, "[[ XML Domain Info (modified) ]]\n");
-			dprintf(3, "%s\n[[ XML END ]]\n", domain_desc);
+			dbg_printf(3, "[[ XML Domain Info (modified) ]]\n");
+			dbg_printf(3, "%s\n[[ XML END ]]\n", domain_desc);
 		} else {
 			printf("Failed getting domain description from "
 			       "libvirt\n");
 		}
 
-		dprintf(2, "[REBOOT] Calling virDomainDestroy\n");
+		dbg_printf(2, "[REBOOT] Calling virDomainDestroy\n");
 		ret = virDomainDestroy(vdp);
 		if (ret < 0) {
 			printf("virDomainDestroy() failed: %d\n", ret);
@@ -298,14 +301,14 @@ do_fence_request_tcp(fence_req_t *req, fence_auth_type_t auth,
 		} else if (domain_desc) {
 			/* Recreate the domain if possible */
 			/* Success */
-			dprintf(2, "Calling virDomainCreateLinux()...\n");
+			dbg_printf(2, "Calling virDomainCreateLinux()...\n");
 			virDomainCreateLinux(vp, domain_desc, 0);
 			free(domain_desc);
 		}
 		break;
 	}
 	
-	dprintf(3, "Sending response to caller...\n");
+	dbg_printf(3, "Sending response to caller...\n");
 	if (write(fd, &response, 1) < 0) {
 		perror("write");
 	}
@@ -540,7 +543,7 @@ xvmd_loop(cman_handle_t ch, void *h, int fd, fence_xvm_args_t *args,
 	virt_list_t *vl = NULL;
 	virt_state_t *dom = NULL;
 
-	vp = virConnectOpen(NULL);
+	vp = virConnectOpen(LIBVIRT_XEN_URI);
 	if (!vp)
 		perror("virConnectOpen");
 
@@ -576,7 +579,7 @@ xvmd_loop(cman_handle_t ch, void *h, int fd, fence_xvm_args_t *args,
 			continue;
 	
 		/* Request and/or timeout: open connection */
-		vp = virConnectOpen(NULL);
+		vp = virConnectOpen(LIBVIRT_XEN_URI);
 		if (!vp) {
 			printf("NOTICE: virConnectOpen(): %s; cannot fence!\n",
 			       strerror(errno));
@@ -669,6 +672,16 @@ xvmd_loop(cman_handle_t ch, void *h, int fd, fence_xvm_args_t *args,
 }
 
 
+void
+sigint_handler(int sig)
+{
+	running = 0;
+}
+
+
+void malloc_dump_table(void);
+
+
 int
 main(int argc, char **argv)
 {
@@ -678,10 +691,15 @@ main(int argc, char **argv)
 	int key_len = 0, x;
 	char *my_options = "dfi:a:p:C:c:k:u?hLXV";
 	cman_handle_t ch = NULL;
-	void *h;
+	void *h = NULL;
 
 	args_init(&args);
 	args_get_getopt(argc, argv, my_options, &args);
+
+	if (!(args.flags & F_NOCCS)) {
+		args_get_ccs(my_options, &args);
+	}
+
 	args_finalize(&args);
 	if (args.debug > 0) {
 		dset(args.debug);
@@ -700,8 +718,8 @@ main(int argc, char **argv)
 
 	if (args.flags & F_VERSION) {
 		printf("%s %s\n", basename(argv[0]), XVM_VERSION);
-#ifdef FENCE_RELEASE_NAME
-		printf("fence release %s\n", FENCE_RELEASE_NAME);
+#ifdef RELEASE_VERSION
+		printf("fence release %s\n", RELEASE_VERSION);
 #endif
 		exit(0);
 	}
@@ -772,7 +790,13 @@ main(int argc, char **argv)
 		return 1;
 	}
 
+
+	signal(SIGINT, sigint_handler);
+	signal(SIGTERM, sigint_handler);
+	signal(SIGQUIT, sigint_handler);
 	xvmd_loop(ch, h, mc_sock, &args, key, key_len);
 
+	//malloc_dump_table();
+
 	return 0;
 }
