@@ -57,7 +57,7 @@ void usage(char *name)
 void version(void)
 {
 	printf("GFS fsck %s (built %s %s)\n",
-	       GFS_RELEASE_NAME, __DATE__, __TIME__);
+	       RELEASE_VERSION, __DATE__, __TIME__);
 	printf("%s\n", REDHAT_COPYRIGHT);
 }
 
