@@ -83,7 +83,6 @@ static int check_metalist(struct gfs2_inode *ip, uint64_t block,
 	}
 	nbh = bread(ip->i_sbd, block);
 
-	/** Attention -- experimental code **/
 	if (gfs2_check_meta(nbh, GFS2_METATYPE_IN)){
 		log_debug("Bad indirect block pointer "
 				  "(points to something that is not an indirect block).\n");
@@ -94,8 +93,6 @@ static int check_metalist(struct gfs2_inode *ip, uint64_t block,
 		}
 	}else  /* blk check ok */
 		*bh = nbh;
-	brelse(nbh, not_updated);
-	/** Attention -- experimental code end **/
 
 	log_debug("Setting %" PRIu64 " (0x%" PRIx64 ") to indirect block.\n",
 			  block, block);
@@ -143,7 +140,6 @@ static int check_eattr_indir(struct gfs2_inode *ip, uint64_t indirect,
 	int ret = 0;
 	struct gfs2_block_query q = {0};
 	struct block_count *bc = (struct block_count *) private;
-	uint64_t block;
 
 	/* This inode contains an eattr - it may be invalid, but the
 	 * eattr attributes points to a non-zero block */
@@ -167,23 +163,27 @@ static int check_eattr_indir(struct gfs2_inode *ip, uint64_t indirect,
 		ret = 1;
 	}
 	else {
-		log_debug("Setting %" PRIu64 " (0x%"
-			  PRIx64 ") to eattr block\n", indirect, indirect);
-		gfs2_block_set(bl, indirect, gfs2_eattr_block);
+		log_debug("Marking inode %" PRIu64 " (0x%"
+			  PRIx64 ") with eattr block\n",
+			  ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
+		/* Mark the inode as having an eattr in the block map
+		   so pass1c can check it. */
+		gfs2_block_mark(bl, ip->i_di.di_num.no_addr, gfs2_eattr_block);
 
 		*bh = bread(sdp, indirect);
-		block = be64_to_cpu(*(*bh)->b_data);
 		if(gfs2_check_meta(*bh, GFS2_METATYPE_IN)) {
 			log_warn("EA indirect block %" PRIu64 " (0x%" PRIx64
-					 ") has incorrect type.\n", block, block);
-			gfs2_block_set(bl, block, gfs2_meta_inval);
+				 ") has incorrect type.\n",
+				 indirect, indirect);
+			gfs2_block_set(bl, indirect, gfs2_meta_inval);
 			ret = 1;
+			brelse(*bh, not_updated);
 		}
 		else {
 			/* FIXME: do i need to differentiate this as an ea_indir? */
 			log_debug("Setting %" PRIu64 " (0x%" PRIx64
-					  ") to indirect block\n", block, block);
-			gfs2_block_set(bl, block, gfs2_indir_blk);
+				  ") to indirect block\n", indirect, indirect);
+			gfs2_block_set(bl, indirect, gfs2_indir_blk);
 			bc->ea_count++;
 		}
 	}
@@ -253,7 +253,7 @@ static int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
 			    void *private)
 {
 	struct gfs2_sbd *sdp = ip->i_sbd;
-	struct gfs2_buffer_head *leaf_bh;
+	struct gfs2_buffer_head *leaf_bh = NULL;
 	int ret = 0;
 	struct gfs2_block_query q = {0};
 	struct block_count *bc = (struct block_count *) private;
@@ -282,6 +282,7 @@ static int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
 				  block, block);
 		gfs2_block_set(bl, block, gfs2_dup_block);
 		bc->ea_count++;
+		ret = 1;
 	}
 	else {
 		leaf_bh = bread(sdp, block);
@@ -293,6 +294,7 @@ static int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
 				 ip->i_di.di_num.no_addr);
 			gfs2_block_set(bl, block, gfs2_meta_inval);
 			ret = -1;
+			brelse(leaf_bh, not_updated);
 		}
 		else {
 			log_debug("Setting block %" PRIu64 " (0x%" PRIx64
@@ -300,7 +302,6 @@ static int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
 			gfs2_block_set(bl, block, gfs2_meta_eattr);
 			bc->ea_count++;
 		}
-		brelse(leaf_bh, not_updated);
 	}
 	*bh = leaf_bh;
 
@@ -462,7 +463,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 	invalidate_metatree.check_data = clear_data;
 	invalidate_metatree.check_leaf = clear_leaf;
 
-	ip = inode_get(sdp, bh);
+	ip = fsck_inode_get(sdp, bh);
 	if (ip->i_di.di_num.no_addr != block) {
 		log_err("Inode #%" PRIu64 " (0x%" PRIx64
 				"): Bad inode address found: %"	PRIu64 " (0x%" PRIx64 ")\n",
@@ -480,7 +481,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 
 	if(gfs2_block_check(bl, block, &q)) {
 		stack;
-		inode_put(ip, f);
+		fsck_inode_put(ip, f);
 		return -1;
 	}
 	if(q.block_type != gfs2_block_free) {
@@ -488,10 +489,10 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 				  block, block);
 		if(gfs2_block_mark(bl, block, gfs2_dup_block)) {
 			stack;
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return -1;
 		}
-		inode_put(ip, f);
+		fsck_inode_put(ip, f);
 		return 0;
 	}
 
@@ -502,12 +503,12 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 				  block, block);
 		if(gfs2_block_set(bl, block, gfs2_inode_dir)) {
 			stack;
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return -1;
 		}
 		if(add_to_dir_list(sdp, block)) {
 			stack;
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return -1;
 		}
 		break;
@@ -516,7 +517,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 				  block, block);
 		if(gfs2_block_set(bl, block, gfs2_inode_file)) {
 			stack;
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return -1;
 		}
 		break;
@@ -525,7 +526,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 				  block, block);
 		if(gfs2_block_set(bl, block, gfs2_inode_lnk)) {
 			stack;
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return -1;
 		}
 		break;
@@ -534,7 +535,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 				  block, block);
 		if(gfs2_block_set(bl, block, gfs2_inode_blk)) {
 			stack;
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return -1;
 		}
 		break;
@@ -543,7 +544,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 				  block, block);
 		if(gfs2_block_set(bl, block, gfs2_inode_chr)) {
 			stack;
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return -1;
 		}
 		break;
@@ -552,7 +553,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 				  block, block);
 		if(gfs2_block_set(bl, block, gfs2_inode_fifo)) {
 			stack;
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return -1;
 		}
 		break;
@@ -561,7 +562,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 				  block, block);
 		if(gfs2_block_set(bl, block, gfs2_inode_sock)) {
 			stack;
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return -1;
 		}
 		break;
@@ -570,15 +571,15 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 				  block, block);
 		if(gfs2_block_set(bl, block, gfs2_meta_inval)) {
 			stack;
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return -1;
 		}
-		inode_put(ip, f);
+		fsck_inode_put(ip, f);
 		return 0;
 	}
 	if(set_link_count(ip->i_sbd, ip->i_di.di_num.no_addr, ip->i_di.di_nlink)) {
 		stack;
-		inode_put(ip, f);
+		fsck_inode_put(ip, f);
 		return -1;
 	}
 
@@ -593,10 +594,10 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 		log_warn("Marking inode invalid\n");
 		if(gfs2_block_set(bl, block, gfs2_meta_inval)) {
 			stack;
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return -1;
 		}
-		inode_put(ip, f);
+		fsck_inode_put(ip, f);
 		return 0;
 	}
 
@@ -612,10 +613,10 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 			log_warn("Marking inode invalid\n");
 			if(gfs2_block_set(bl, block, gfs2_meta_inval)) {
 				stack;
-				inode_put(ip, f);
+				fsck_inode_put(ip, f);
 				return -1;
 			}
-			inode_put(ip, f);
+			fsck_inode_put(ip, f);
 			return 0;
 		}
 	}
@@ -624,7 +625,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 
 	error = check_metatree(ip, &pass1_fxns);
 	if(error < 0) {
-		inode_put(ip, f);
+		fsck_inode_put(ip, f);
 		return 0;
 	}
 	if(error > 0) {
@@ -632,7 +633,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 		/* FIXME: Must set all leaves invalid as well */
 		check_metatree(ip, &invalidate_metatree);
 		gfs2_block_set(bl, ip->i_di.di_num.no_addr, gfs2_meta_inval);
-		inode_put(ip, f);
+		fsck_inode_put(ip, f);
 		return 0;
 	}
 
@@ -660,7 +661,7 @@ int handle_di(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh,
 					ip->i_di.di_num.no_addr);
 	}
 
-	inode_put(ip, f);
+	fsck_inode_put(ip, f);
 	return 0;
 }
 
@@ -766,8 +767,10 @@ int pass1(struct gfs2_sbd *sbp)
 			if (gfs2_next_rg_meta(rgd, &block, first))
 				break;
 			warm_fuzzy_stuff(block);
-			if (fsck_abort) /* if asked to abort */
+			if (fsck_abort) { /* if asked to abort */
+				gfs2_rgrp_relse(rgd, not_updated);
 				return 0;
+			}
 			if (skip_this_pass) {
 				printf("Skipping pass 1 is not a good idea.\n");
 				skip_this_pass = FALSE;
