@@ -32,28 +32,6 @@
 #define CPIN_64(s1, s2, member) {(s1->member) = be64_to_cpu((s2->member));}
 #define CPOUT_64(s1, s2, member) {(s2->member) = cpu_to_be64((s1->member));}
 
-/**
- * print_array - Print out an array of bytes
- * @title: what to print before the array
- * @buf: the array
- * @count: the number of bytes
- *
- */
-
-static void print_array(char *title, unsigned char *buf, int count)
-{
-	int x;
-
-	printf("  %s =\n", title);
-	for (x = 0; x < count; x++) {
-		printf("%.2X ", (unsigned char)buf[x]);
-		if (x % 16 == 15)
-			printf("\n");
-	}
-	if (x % 16)
-		printf("\n");
-}
-
 /*
  * gfs2_xxx_in - read in an xxx struct
  * first arg: the cpu-order structure
@@ -252,6 +230,8 @@ void gfs2_quota_in(struct gfs2_quota *qu, char *buf)
 	CPIN_64(qu, str, qu_limit);
 	CPIN_64(qu, str, qu_warn);
 	CPIN_64(qu, str, qu_value);
+	CPIN_32(qu, str, qu_ll_next);
+	CPIN_08(qu, str, qu_reserved, 60);
 }
 
 void gfs2_quota_out(struct gfs2_quota *qu, char *buf)
@@ -261,7 +241,8 @@ void gfs2_quota_out(struct gfs2_quota *qu, char *buf)
 	CPOUT_64(qu, str, qu_limit);
 	CPOUT_64(qu, str, qu_warn);
 	CPOUT_64(qu, str, qu_value);
-	memset(qu->qu_reserved, 0, sizeof(qu->qu_reserved));
+	CPOUT_32(qu, str, qu_ll_next);
+	CPOUT_08(qu, str, qu_reserved, 60);
 }
 
 void gfs2_quota_print(struct gfs2_quota *qu)
@@ -405,7 +386,7 @@ void gfs2_dirent_print(struct gfs2_dirent *de, char *name)
 
 	memset(buf, 0, GFS2_FNAMESIZE + 1);
 	memcpy(buf, name, de->de_name_len);
-	printf("  name = %s\n", buf);
+	print_it("  name", "%s", NULL, buf);
 }
 
 void gfs2_leaf_in(struct gfs2_leaf *lf, char *buf)
@@ -481,7 +462,7 @@ void gfs2_ea_header_print(struct gfs2_ea_header *ea, char *name)
 
 	memset(buf, 0, GFS2_EA_MAX_NAME_LEN + 1);
 	memcpy(buf, name, ea->ea_name_len);
-	printf("  name = %s\n", buf);
+	print_it("  name", "%s", NULL, buf);
 }
 
 void gfs2_log_header_in(struct gfs2_log_header *lh, char *buf)
