@@ -238,6 +238,7 @@ void app_start(app_t *a)
 		type = NODE_FAILED;
 	else {
 		/* report error */
+		type = -1;
 	}
 
 	/* start <name> <event_nr> <type> <count> <memb0> <memb1>... */
@@ -552,19 +553,20 @@ static int do_get_group(int ci, int argc, char **argv)
 
 static int do_dump(int fd)
 {
-	int rv, len = DUMP_SIZE;
+	int len;
 
 	if (dump_wrap) {
 		len = DUMP_SIZE - dump_point;
-		rv = do_write(fd, dump_buf + dump_point, len);
-		if (rv < 0)
-			log_print("dump write error %d errno %d", rv, errno);
+		do_write(fd, dump_buf + dump_point, len);
+		len = dump_point;
+	} else
 		len = dump_point;
-	}
 
-	rv = do_write(fd, dump_buf, len);
-	if (rv < 0)
-		log_print("dump write error %d errno %d", rv, errno);
+	/* NUL terminate the debug string */
+	dump_buf[dump_point] = '\0';
+
+	do_write(fd, dump_buf, len);
+
 	return 0;
 }
 
@@ -666,6 +668,7 @@ static void process_connection(int ci)
 
 	case DO_DUMP:
 		do_dump(client[ci].fd);
+		close(client[ci].fd);
 		break;
 
 	case DO_LOG:
@@ -761,7 +764,7 @@ static int loop(void)
 		for (i = 0; i <= client_maxi; i++) {
 			if (client[i].fd < 0)
 				continue;
-			if (pollfd[i].revents & POLLHUP) {
+			if (pollfd[i].revents & (POLLERR | POLLHUP | POLLNVAL)) {
 				deadfn = client[i].deadfn;
 				deadfn(i);
 			} else if (pollfd[i].revents & POLLIN) {
@@ -904,6 +907,18 @@ static void decode_arguments(int argc, char **argv)
 	}
 }
 
+void set_oom_adj(int val)
+{
+	FILE *fp;
+
+	fp = fopen("/proc/self/oom_adj", "w");
+	if (!fp)
+		return;
+
+	fprintf(fp, "%i", val);
+	fclose(fp);
+}
+
 void set_scheduler(void)
 {
 	struct sched_param sched_param;
@@ -939,8 +954,7 @@ void bail_with_log(int sig)
 		write(fd, now_ascii, strlen(now_ascii));
 		write(fd, " groupd segfault log follows:\n", 30);
 		close(fd);
-	}
-	else
+	} else
 		perror(LOG_FILE);
 	if (sig == SIGSEGV)
 		exit(0);
@@ -960,10 +974,12 @@ int main(int argc, char *argv[])
 
 	signal(SIGSEGV, bail_with_log);
 	signal(SIGUSR1, bail_with_log);
+
 	if (!groupd_debug_opt)
 		daemonize();
 
 	set_scheduler();
+	set_oom_adj(-16);
 
 	pollfd = malloc(NALLOC * sizeof(struct pollfd));
 	if (!pollfd)
